# - Find the CUDD LP solver.
# This code defines the following variables:
#
#  CUDD_FOUND                 - TRUE if CUDD was found.
#  CUDD_INCLUDE_DIRS          - Full paths to all include dirs.
#  CUDD_LIBRARIES             - Full paths to all libraries.
#  CUDD_RUNTIME_LIBRARY       - Full path to the dll file on windows
#
# Usage:
#  find_package(cplex)
#
# The location of CUDD can be specified using the environment variable
# or cmake parameter DOWNWARD_CUDD_ROOT.
#
# Note that the standard FIND_PACKAGE features are supported
# (QUIET, REQUIRED, etc.).

find_path(CUDD_INCLUDE_DIRS
    NAMES
    cudd.h
    HINTS
    $ENV{DOWNWARD_CUDD_ROOT}
    ${DOWNWARD_CUDD_ROOT}
    ${CMAKE_CURRENT_SOURCE_DIR}/cudd-3.0.0
    PATH_SUFFIXES
    cudd
)

set(CUDD_LIBRARY_PATH_SUFFIX "cudd/.libs/")

find_library(CUDD_LIBRARY_RELEASE
    NAMES
    cudd
    HINTS
    $ENV{DOWNWARD_CUDD_ROOT}
    ${DOWNWARD_CUDD_ROOT}
    ${CMAKE_CURRENT_SOURCE_DIR}/cudd-3.0.0
    PATH_SUFFIXES
    ${CUDD_LIBRARY_PATH_SUFFIX}
)

find_library(CUDD_LIBRARY_DEBUG
    NAMES
    cudd
    HINTS
    $ENV{DOWNWARD_CUDD_ROOT}
    ${DOWNWARD_CUDD_ROOT}
    ${CMAKE_CURRENT_SOURCE_DIR}/cudd-3.0.0
    PATH_SUFFIXES
    ${CUDD_LIBRARY_PATH_SUFFIX}
)

if(CUDD_LIBRARY_RELEASE OR CUDD_LIBRARY_DEBUG)
    set(CUDD_LIBRARIES
        optimized ${CUDD_LIBRARY_RELEASE} ${CMAKE_THREAD_LIBS_INIT}
        debug ${CUDD_LIBRARY_DEBUG} ${CMAKE_THREAD_LIBS_INIT}
    )
endif()

# Check if everything was found and set CUDD_FOUND.
include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(
    Cudd
    REQUIRED_VARS CUDD_INCLUDE_DIRS CUDD_LIBRARIES
)

mark_as_advanced(
    CUDD_INCLUDE_DIRS CUDD_LIBRARIES CUDD_LIBRARY_PATH_SUFFIX
)
