#ifndef COST_SATURATION_TYPES_H
#define COST_SATURATION_TYPES_H

#include <limits>
#include <memory>
#include <vector>
#include <functional>

namespace cost_saturation {
class Abstraction;
class CostPartitionedHeuristic;

// Positive infinity. The name "INFINITY" is taken by an ISO C99 macro.
const int INF = std::numeric_limits<int>::max();

using CPCollectionCallback = std::function<void(unsigned, bool, const std::vector<int>&)>;

struct CallbackCaller {
    enum Modes {
        NEW = 0,
        IN_CONSTRUCTION = 1,
        POP_BACK = 2,
        POP_PREVIOUS = 3,
    };
    unsigned mode;
    CPCollectionCallback* callback;
    CallbackCaller(CPCollectionCallback* callback) : mode(0), callback(callback) {}
    void operator()(bool before, const std::vector<int>& cp) {
        if (callback) (*callback)(mode, before, cp);
        mode = IN_CONSTRUCTION;
    }
    void pop_back()
    {
        static std::vector<int> dummy;
        mode = POP_BACK;
        if (callback) (*callback)(mode, false, dummy);
    }
    void pop_previous()
    {
        static std::vector<int> dummy;
        mode = POP_PREVIOUS;
        if (callback) (*callback)(mode, false, dummy);
    }
    static CallbackCaller ignore() {
        return CallbackCaller(NULL);
    }
};

using Abstractions = std::vector<std::unique_ptr<Abstraction>>;
using CPFunction = std::function<CostPartitionedHeuristic(
                                     const Abstractions &,
                                     const std::vector<int> &,
                                     const std::vector<int> &,
                                     bool,
                                     CallbackCaller&)>;
using CPHeuristics = std::vector<CostPartitionedHeuristic>;
using Order = std::vector<int>;

}

#endif
