#include "backward_hyperabstraction_heuristic.h"
#include "utils.h"

#include "../global_state.h"
#include "../globals.h"
#include "../option_parser.h"
#include "../plugin.h"
#include "../utils/timer.h"
#include "../task_tools.h"

#include <iostream>

namespace hyperabstraction {

BackwardHyperabstractionHeuristic::BackwardHyperabstractionHeuristic(const options::Options& opts)
    : Heuristic(opts)
    , m_generator(get_hyperabstraction_generator_from_options(opts))
    , m_abstraction(nullptr)
{
    std::cout << "Initializing backward hyperabstraction heuristic..." << std::endl;
    ::verify_no_axioms(task_proxy); 
    ::verify_no_conditional_effects(task_proxy); 
    
    HyperabstractionGeneratorResult<Hyperabstraction> r(
            get_hyperabstraction_from_options(opts, m_generator, false));
    m_abstraction = r.abstraction;
    m_sink_nodes = std::move(r.goal_nodes);
}

int
BackwardHyperabstractionHeuristic::compute_heuristic(const GlobalState& state)
{
    static std::vector<int> distances(m_abstraction->num_nodes());
    m_generator->get_nodes(m_source_nodes, state);
    m_abstraction->compute_distances(distances, m_source_nodes);
    int res = 0;
    for (int i = m_sink_nodes.size() - 1; i >= 0; i--) {
        res = std::max(res, distances[m_sink_nodes[i]]);
    }
    m_source_nodes.clear();
    return res == INF ? DEAD_END : res;
}

void
BackwardHyperabstractionHeuristic::add_options_to_parser(options::OptionParser& parser)
{
    add_hyperabstraction_options_to_parser(parser);
    Heuristic::add_options_to_parser(parser);
}

}

static Heuristic*
_parse(options::OptionParser& parser)
{
    hyperabstraction::BackwardHyperabstractionHeuristic::add_options_to_parser(parser);
    options::Options opts = parser.parse();
    if (!parser.dry_run()) {
        return new hyperabstraction::BackwardHyperabstractionHeuristic(opts);
    }
    return NULL;
}

static Plugin<Heuristic> _plugin("hyb", _parse);
