#include "bidirectional_hyperabstraction_heuristic.h"
#include "generic_hyperabstraction_generator.h"
#include "utils.h"

#include "../global_state.h"
#include "../option_parser.h"
#include "../plugin.h"
#include "../utils/timer.h"

#include <algorithm>

namespace hyperabstraction {

BidirectionalHyperabstractionHeuristic::BidirectionalHyperabstractionHeuristic(
        const options::Options& opts)
    : Heuristic(opts)
    , m_generator(get_hyperabstraction_generator_from_options(opts))
    , m_fw_abstraction(nullptr)
    , m_bw_abstraction(nullptr)
{
    std::cout << "Initializing bidirectional hyperabstraction heuristic..." << std::endl;

    auto construction = get_bidirectional_hyperabstractions_from_options(opts, m_generator);

    auto& fw = construction.first;
    m_fw_abstraction = fw.abstraction;
    m_fw_init = std::move(fw.initial_nodes);
    //fw.statistics.print();

    auto& bw = construction.second;
    m_bw_abstraction = bw.abstraction;
    m_bw_goal = std::move(bw.goal_nodes);
    // bw.statistics.print();

    //unsigned num_nodes = std::max(m_fw_abstraction->num_nodes(),
    //                              m_bw_abstraction->num_nodes());
    m_num_actual_nodes = m_generator->num_nodes() + 1;

    // if (opts.get<bool>("prune_unreachable")) {
    //     utils::Timer t;
    //     std::cout << "Computing unreachable nodes..." << std::endl;
    //     std::vector<bool> reachable(num_nodes);
    //     m_fw_abstraction->compute_reachable(reachable, m_fw_init);
    //     compute_bidirectional_reachability(reachable,
    //                                        m_fw_init,
    //                                        m_bw_goal,
    //                                        fw.goal_nodes,
    //                                        m_num_actual_nodes,
    //                                        *m_fw_abstraction,
    //                                        *m_bw_abstraction);
    //     unsigned fwt = m_fw_abstraction->num_transitions();
    //     unsigned bwt = m_bw_abstraction->num_transitions();

    //     std::vector<bool> reachable2(num_nodes);
    //     std::fill(reachable.begin() + m_num_actual_nodes, reachable.end(), false);
    //     unsigned unr = m_fw_abstraction->prune_unreachable(reachable);
    // }

    std::cout << "Precomputing forward hyperabstraction distances..." << std::endl;
    m_fw_distances.resize(m_fw_abstraction->num_nodes());
    m_fw_abstraction->compute_distances(m_fw_distances, m_fw_init);

    if (m_bw_abstraction->num_nodes() > m_fw_abstraction->num_nodes()) {
        m_fw_distances.resize(m_bw_abstraction->num_nodes(), 0);
    }

}

int
BidirectionalHyperabstractionHeuristic::compute_heuristic(const GlobalState& state)
{
    static std::vector<int> distances(m_fw_distances.size());
    assert(m_state_nodes.empty());
    m_generator->get_nodes(m_state_nodes, state);
    distances = m_fw_distances;
    int res = compute_bidirectional_distances(
            distances,
            m_fw_init,
            m_bw_goal,
            m_state_nodes,
            m_num_actual_nodes,
            *m_fw_abstraction,
            *m_bw_abstraction);
    m_state_nodes.clear();
    return res == INF ? DEAD_END : res;
}

void
BidirectionalHyperabstractionHeuristic::add_options_to_parser(options::OptionParser& parser)
{
    add_hyperabstraction_options_to_parser(parser);
    Heuristic::add_options_to_parser(parser);
}

}

static Plugin<Heuristic> _plugin(
        "hybi",
        options::parse<Heuristic, hyperabstraction::BidirectionalHyperabstractionHeuristic>);

