#include "cartesian_hyperabstraction_generator.h"

#include "../abstract_task.h"
#include "../global_state.h"

#include "../option_parser.h"
#include "../plugin.h"

#include "../cegar/cost_saturation.h"
#include "../cegar/abstract_state.h"
#include "../cegar/transition_system.h"

#include "../task_proxy.h"

#include <set>
#include <algorithm>
#include <fstream>

namespace hyperabstraction {

CartesianHyperabstractionGenerator::CartesianHyperabstractionGenerator(const options::Options& opts)
    : HyperabstractionGenerator(opts)
{
    std::shared_ptr<cegar::CostSaturation> cegar = opts.get<std::shared_ptr<cegar::CostSaturation> >("abstraction");
    std::vector<std::unique_ptr<cegar::Abstraction> > abstractions = cegar->extract_abstractions();
    m_node_mappers.reserve(abstractions.size());
    unsigned sum = 0;
    // std::vector<std::shared_ptr<cegar::TransitionSystem> > ts = 
    //     cegar->extract_transition_systems();
    for (unsigned i = 0; i < abstractions.size(); i++) {

        // std::vector<int> x = opts.get<std::vector<int>>("x");
        // if (!std::count(x.begin(), x.end(), (int)i)) {
        //     continue;
        // }

        m_node_mappers.emplace_back(m_task, *abstractions[i], sum);
        sum += m_node_mappers.back().num_nodes();
        // std::cout << "cegar[" << i << "] -> "
        //           << abstractions[i]->get_num_states() << " states" << std::endl;

        // std::ofstream out; out.open("cegar.dot");
        // assert(!ts.empty());
        // ts[0]->dump(out);
        // out.close();

        // std::vector<int> operator_costs(m_task->get_num_operators());
        // for (int op = operator_costs.size() - 1; op >= 0; op--) {
        //     operator_costs[op] = m_task->get_operator_cost(op, false);
        // }
        // TaskProxy task_proxy(*m_task);
        // abstractions[i]->set_operator_costs(operator_costs);
        // std::cout << "I = " << abstractions[i]->get_refinement_hierarchy()->get_local_state_id(task_proxy.get_initial_state()) << std::endl;
        // std::cout << "G = [";
        // bool isfirst = true;
        // for (const auto& a : abstractions[i]->get_goals()) {
        //     std::cout << (isfirst ? "" : ", ") << a->get_node()->get_state_id();
        //     isfirst = false;
        // }
        // std::cout << "]" << std::endl;
        // std::cout << "Value: " << abstractions[i]->get_h_value_of_initial_state() << std::endl;



    }
    cegar->transfer_back_abstractions(std::move(abstractions));
}

HyperabstractionGeneratorResult<Hyperabstraction>
CartesianHyperabstractionGenerator::generate_backward()
{
    return generate<Hyperabstraction, false>();
}

HyperabstractionGeneratorResult<LabeledHyperabstraction>
CartesianHyperabstractionGenerator::generate_backward_labeled()
{
    return generate<LabeledHyperabstraction, false>();
}

HyperabstractionGeneratorResult<UpdatableHyperabstraction>
CartesianHyperabstractionGenerator::generate_backward_updatable()
{
    return generate<UpdatableHyperabstraction, false>();
}

HyperabstractionGeneratorResult<Hyperabstraction>
CartesianHyperabstractionGenerator::generate_forward()
{
    return generate<Hyperabstraction, true>();
}

HyperabstractionGeneratorResult<LabeledHyperabstraction>
CartesianHyperabstractionGenerator::generate_forward_labeled()
{
    return generate<LabeledHyperabstraction, true>();
}

HyperabstractionGeneratorResult<UpdatableHyperabstraction>
CartesianHyperabstractionGenerator::generate_forward_updatable()
{
    return generate<UpdatableHyperabstraction, true>();
}

void
CartesianHyperabstractionGenerator::get_nodes(std::vector<Node>& nodes,
                                        const GlobalState& state)
{
    for (int i = m_node_mappers.size() - 1; i >= 0; i--) {
#ifndef NDEBUG
        assert(m_node_mappers[i].get_compatible_nodes(nodes, state, 1));
#else
        m_node_mappers[i].get_compatible_nodes(nodes, state, 1);
#endif
    }
}

unsigned
CartesianHyperabstractionGenerator::num_nodes() const
{
    unsigned res = 0;
    for (int i = m_node_mappers.size() - 1; i >= 0; i--) {
        res += m_node_mappers[i].num_nodes();
    }
    return res;
}

void
CartesianHyperabstractionGenerator::add_options_to_parser(options::OptionParser& parser)
{
    parser.add_option<std::shared_ptr<cegar::CostSaturation> >(
            "abstraction",
            "",
            "cegar([landmarks(combine_facts=false), goals()], compress_cartesian_states=false)");
    // parser.add_list_option<int>("x");
    HyperabstractionGenerator::add_options_to_parser(parser);
}

}



#include <set>

namespace hyperabstraction {

CartesianNodeMapper::CartesianNodeMapper(
        const std::shared_ptr<AbstractTask> task,
        const cegar::Abstraction& abstraction,
        unsigned offset)
    : m_task(task)
    , m_offset(offset)
    , m_num_nodes(abstraction.get_num_states())
    , m_state_lookup(abstraction.get_refinement_hierarchy())
{
#if 0
    class MyLess {
        const std::shared_ptr<AbstractTask> task;
    public:
        MyLess(const std::shared_ptr<AbstractTask> task) : task(task) {}
        bool operator()(const cegar::AbstractState*x, const cegar::AbstractState* y) const
        {
            const cegar::Domains& s1 = x->get_domains();
            const cegar::Domains& s2 = y->get_domains();
            for (int var = 0; var < task->get_num_variables(); var++) {
                for (int val = 0; val < task->get_variable_domain_size(var); val++) {
                    if (s1.test(var, val) && !s2.test(var, val)) {
                        return true;
                    } else if (!s1.test(var, val) && s2.test(var, val)) {
                        return false;
                    }
                }
            }
            return false;
        }
    };
    MyLess l(task);
    std::set<const cegar::AbstractState *, MyLess> myset(l); //(MyLess(task));
    for (const cegar::AbstractState* s : abstraction.get_states()) {
        myset.insert(s);
    }
    int id = 0;
    for (auto it = myset.begin(); it != myset.end(); it++) {
        (*it)->get_node()->set_state_id(id);
        m_cartesian_states.push_back((*it)->get_domains());
        id++;
    }
    return;
#endif

#ifndef NDEBUG
    std::shared_ptr<AbstractTask> a_task = abstraction.get_underlying_task();
    assert(a_task->get_num_variables() == task->get_num_variables());
    for (int var = task->get_num_variables() - 1; var >= 0; var--) {
        assert(task->get_variable_domain_size(var) == a_task->get_variable_domain_size(var));
    }
#endif
    const cegar::AbstractStates& states_set = abstraction.get_states();
    m_cartesian_states.reserve(abstraction.get_num_states());
    for (auto it = states_set.begin(); it != states_set.end(); it++) {
        assert((*it)->get_node()->get_state_id() == (int) m_cartesian_states.size());
        m_cartesian_states.push_back((*it)->get_domains());
#ifndef NDEBUG
        // std::cout << "state#" << (m_cartesian_states.size() - 1) << " = " << m_cartesian_states.back() << std::endl;
        const cegar::Domains& state1 = m_cartesian_states.back();
        const cegar::Domains& state2 = (*it)->get_domains();
        assert(state2.count(task->get_num_variables() - 1));
        assert(state1.count(task->get_num_variables() - 1));
#endif
    }

// #ifndef NDEBUG
//     std::function<void(const cegar::Node *, std::set<int>&)> verify_distinct;
//     verify_distinct =
//         [&](const cegar::Node* node, std::set<int>& reachable) {
//             if (!node->is_split()) {
//                 reachable.insert(node->get_state_id());
//                 return;
//             }
//             const cegar::Node* right = node->get_right_child();
//             const cegar::Node* left = node->get_left_child();
//             while (left->is_split() && left->get_right_child() == right) {
//                 left = left->get_left_child();
//             }
//             std::set<int> reachable_left;
//             std::set<int> reachable_right;
//             verify_distinct(left, reachable_left);
//             verify_distinct(right, reachable_right);
//             std::set<int> i;
//             std::set_intersection(reachable_left.begin(), reachable_left.end(),
//                                   reachable_right.begin(), reachable_right.end(),
//                                   std::inserter(i, i.begin()));
//             assert(i.empty());
//             reachable.insert(reachable_left.begin(), reachable_left.end());
//             reachable.insert(reachable_right.begin(), reachable_right.end());
//         };
//     std::set<int> r;
//     verify_distinct(m_state_lookup->get_root(), r);
// #endif
}

bool
CartesianNodeMapper::get_compatible_nodes(
        std::vector<Node>& result,
        const std::vector<int>& state,
        unsigned max_nodes) const
{
    return get_compatible_nodes_vector(result, state, max_nodes);
}

bool
CartesianNodeMapper::get_compatible_nodes(
        std::vector<Node>& result,
        const GlobalState& state,
        unsigned max_nodes) const
{
    return get_compatible_nodes_vector(result, state, max_nodes);
}

bool
CartesianNodeMapper::get_compatible_nodes(
        std::vector<Node>& result,
        const cegar::Domains& cartesian_state,
        unsigned max_nodes) const
{
    auto is_defined = [&cartesian_state](const int& var, const int& val) {
        return cartesian_state.test(var, val);
    };
    auto num_defined = [&cartesian_state](const int& var) {
        return (unsigned) cartesian_state.count(var);
    };
    unsigned previous = result.size();
    max_nodes += previous;
    // std::cout << "get_compatible_nodes(" << cartesian_state << ")" << std::endl;
    auto callback = [&](const Node& node) {
#ifndef NDEBUG
        // std::cout << "  => leaf=" << m_cartesian_states[node] << std::endl;
        for (int var = 0; var < m_task->get_num_variables(); var++) {
            assert(m_cartesian_states[node].intersects(cartesian_state, var));
        }
#endif
        if (result.size() == max_nodes) {
            return false;
        }
        result.push_back(m_offset + node);
        return true;
    };
    if (traverse(m_state_lookup->get_root(), is_defined, num_defined, callback)) {
        // std::cout << std::endl;
        return true;
    }
        // std::cout << std::endl;
    result.resize(previous);
    return false;
}

unsigned
CartesianNodeMapper::get_num_compatible_nodes(
        const std::vector<int>& partial_state,
        unsigned max_nodes) const
{
    auto is_defined = [&partial_state](const int& var, const int& val) {
        return (partial_state[var] == -1) || (partial_state[var] == val);
    };
    auto num_defined = [&](const int& var) {
        return (unsigned) (partial_state[var] == -1 ? m_task->get_variable_domain_size(var) : 1);
    };
    unsigned num = 0;
    auto callback = [&num,&max_nodes](const Node& ) {
        return ++num == max_nodes;
    };
    traverse(m_state_lookup->get_root(), is_defined, num_defined, callback);
    return num;
}


unsigned
CartesianNodeMapper::get_num_compatible_nodes(
        const cegar::Domains& cartesian_state,
        unsigned max_nodes) const
{
    auto is_defined = [&cartesian_state](const int& var, const int& val) {
        return cartesian_state.test(var, val);
    };
    auto num_defined = [&cartesian_state](const int& var) {
        return (unsigned) cartesian_state.count(var);
    };
    unsigned num = 0;
    auto callback = [&num,&max_nodes](const Node& ) {
        return ++num == max_nodes;
    };
    traverse(m_state_lookup->get_root(), is_defined, num_defined, callback);
    return num;
}

cegar::Domains
CartesianNodeMapper::compute_intersection(const Node& node,
                                                                 const std::vector<int>& state) const
{
    return m_cartesian_states[node - m_offset].intersection(state);
}

cegar::Domains
CartesianNodeMapper::compute_intersection(const Node& node,
                                                                 const cegar::Domains& state) const
{
    return m_cartesian_states[node - m_offset].intersection(state);
}

unsigned
CartesianNodeMapper::num_nodes() const
{
    return m_num_nodes;
}


}

static std::shared_ptr<hyperabstraction::HyperabstractionGenerator>
_parse(options::OptionParser& parser)
{
    hyperabstraction::CartesianHyperabstractionGenerator::add_options_to_parser(parser);
    Options opts = parser.parse();
    if (!parser.dry_run()) {
        opts.set<bool>("compress_cartesian_states", false);
        return std::make_shared<hyperabstraction::CartesianHyperabstractionGenerator>(opts);
    }
    return nullptr;
}

static PluginShared<hyperabstraction::HyperabstractionGenerator> _plugin(
        "cartesian",
        _parse);

