#ifndef COMBINED_HYPERABSTRACTION_GENERATOR_H
#define COMBINED_HYPERABSTRACTION_GENERATOR_H

#include "hyperabstraction_generator.h"
#include "pdb_hyperabstraction_generator.h"
#include "cartesian_hyperabstraction_generator.h"

namespace hyperabstraction {

class CombinedHyperabstractionGenerator : public HyperabstractionGenerator {
public:
    CombinedHyperabstractionGenerator(const options::Options& opts);

    virtual HyperabstractionGeneratorResult<Hyperabstraction> generate_backward() override;
    virtual HyperabstractionGeneratorResult<LabeledHyperabstraction> generate_backward_labeled() override;
    virtual HyperabstractionGeneratorResult<UpdatableHyperabstraction> generate_backward_updatable() override;
    virtual HyperabstractionGeneratorResult<Hyperabstraction> generate_forward() override;
    virtual HyperabstractionGeneratorResult<LabeledHyperabstraction> generate_forward_labeled() override;
    virtual HyperabstractionGeneratorResult<UpdatableHyperabstraction> generate_forward_updatable() override;

    virtual void get_nodes(std::vector<Node>& nodes, const GlobalState& state) override;
    virtual unsigned num_nodes() const override;

    static void add_options_to_parser(options::OptionParser& parser);
protected:

    template<typename Abstraction, bool Direction>
    HyperabstractionGeneratorResult<Abstraction> generate();

    std::vector<PDBNodeMapper> m_pdbs;
    std::vector<CartesianNodeMapper> m_cartesians;
};

}


namespace hyperabstraction {

template<typename Abstraction, bool Direction>
HyperabstractionGeneratorResult<Abstraction>
CombinedHyperabstractionGenerator::generate()
{
    return generate_hyperabstraction<Abstraction>(
            m_task,
            Direction,
            c_tau_max_nodes,
            c_tau_max_nodes_goal,
            c_cache_tau_nodes,
            c_transition_pruning,
            false,
            c_transition_enumeration_depth,
            c_goal_enumeration_depth,
            c_enumeration_max_nodes,
            m_pdbs,
            m_cartesians);
}


}

#endif
