#ifndef COST_PARTITIONING_H
#define COST_PARTITIONING_H

#include "types.h"
#include "utils.h"
#include "hyperabstraction.h"

#include <vector>

namespace hyperabstraction {

using CostPartitioning = std::vector<std::vector<int> >;

struct CostPartitioningCollection {
    CostPartitioningCollection();
    CostPartitioningCollection(std::vector<CostPartitioning>&& collection);
    CostPartitioningCollection(const CostPartitioning& cps,
                               const std::vector<std::vector<unsigned> >& collection);
    CostPartitioningCollection(const CostPartitioningCollection& other) = default;
    CostPartitioningCollection(CostPartitioningCollection&& other) = default;
    CostPartitioningCollection& operator=(CostPartitioningCollection&& other) = default;
    CostPartitioningCollection& operator=(std::vector<CostPartitioning>&& collection);

    void update_distances(LabeledHyperabstraction& abstraction,
                          const std::vector<Node>& from);

    int get_max_sum(const std::vector<Node>& nodes) const;

    unsigned get_num_partitions() const;
    unsigned get_num_cost_functions() const;

    CostPartitioning m_cost_partitions;
    std::vector<std::vector<unsigned> > m_collection;
    std::vector<std::vector<int> > m_distances;
};

}

#endif
