#include "cost_partitioning_finder.h"

#include "../plugin.h"
#include "../option_parser.h"
#include "../operator_cost.h"
#include "../abstract_task.h"
#include "cost_partitioning.h"

#include "../utils/hash.h"

#include <set>
#include <unordered_set>
#include <algorithm>

namespace hyperabstraction {

CostPartitioningFinder::CostPartitioningFinder(const options::Options& opts)
    : m_task(get_task_from_options(opts))
{}

int
CostPartitioningFinder::compute_cost_partitioning_and_get_distance(
        LabeledHyperabstraction& abstraction,
        const std::vector<Node>& source,
        const std::vector<Node>& sink)
{
    CostPartitioningCollection collection(compute_cost_partitioning(
                abstraction,
                source,
                sink));
    collection.update_distances(abstraction, source);
    return collection.get_max_sum(sink);
}

void
CostPartitioningFinder::add_options_to_parser(options::OptionParser& parser)
{
    ::add_cost_type_option_to_parser(parser);
    parser.add_option<std::shared_ptr<AbstractTask>>(
        "transform",
        "Optional task transformation for the heuristic. "
        "Currently only adapt_costs is available.",
        OptionParser::NONE);
}

}


static PluginTypePlugin<hyperabstraction::CostPartitioningFinder> _plugin_type("CostPartitioningFinder", ""); 

