#include "forward_hyperabstraction_heuristic.h"
#include "generic_hyperabstraction_generator.h"
#include "utils.h"
#include "../global_state.h"
#include "../globals.h"
#include "../option_parser.h"
#include "../plugin.h"
#include "../utils/timer.h"
#include "../task_tools.h"
#include <iostream>
#include <fstream>

namespace hyperabstraction {

ForwardHyperabstractionHeuristic::ForwardHyperabstractionHeuristic(const options::Options& opts)
    : Heuristic(opts)
    , m_generator(get_hyperabstraction_generator_from_options(opts))
{
    std::cout << "Initializing forward hyperabstraction heuristic..." << std::endl;
    ::verify_no_axioms(task_proxy); 
    ::verify_no_conditional_effects(task_proxy); 

    auto r = get_hyperabstraction_from_options(opts, m_generator, true);
    std::shared_ptr<Hyperabstraction> abstraction = r.abstraction;

    utils::Timer t;
    m_distances.resize(abstraction->num_nodes());
    abstraction->compute_distances(m_distances, r.initial_nodes);
    std::cout << "Computed distances in " << t << std::endl;

    // auto dump = [](const std::vector<Node>& nodes) {
    //     std::cout << "[";
    //     for (unsigned i = 0; i < nodes.size(); i++) {
    //         std::cout << (i > 0 ? ", " : "") << nodes[i];
    //     }
    //     std::cout << "]" << std::endl;
    // };
    // std::cout << "I = "; dump(r.goal_nodes);
    // std::cout << "G = "; dump(r.initial_nodes);

    // std::cout << "Value: " << get_max_distance(m_distances, r.goal_nodes) << std::endl;


    // exit(1);
}

int
ForwardHyperabstractionHeuristic::compute_heuristic(const GlobalState& state)
{
    m_generator->get_nodes(m_satisfied_nodes, state);
    int res = 0;
    for (int i = m_satisfied_nodes.size() - 1; i >= 0; i--) {
        res = std::max(res, m_distances[m_satisfied_nodes[i]]);
    }
    m_satisfied_nodes.clear();
    return res == INF ? DEAD_END : res;
}

void
ForwardHyperabstractionHeuristic::add_options_to_parser(options::OptionParser& parser)
{
    add_hyperabstraction_options_to_parser(parser);
    Heuristic::add_options_to_parser(parser);
}

}

static Heuristic*
_parse(options::OptionParser& parser)
{
    hyperabstraction::ForwardHyperabstractionHeuristic::add_options_to_parser(parser);
    options::Options opts = parser.parse();
    if (!parser.dry_run()) {
        return new hyperabstraction::ForwardHyperabstractionHeuristic(opts);
    }
    return NULL;
}

static Plugin<Heuristic> _plugin("hyf", _parse);
