#ifndef GENERIC_HYPERABSTRACTION_GENERATOR_H
#define GENERIC_HYPERABSTRACTION_GENERATOR_H

#include "types.h"
#include "hyperabstraction_generator_statistics.h"

#include <memory>

class AbstractTask;

namespace hyperabstraction {

enum class TransitionPruningMethod {
    DISABLED = 0,
    DUPLICATES = 1,
    SUBSUMED = 2,
};

template<typename Hyperabstraction>
struct HyperabstractionGeneratorResult {
    HyperabstractionGeneratorStatistics statistics;
    std::shared_ptr<Hyperabstraction> abstraction;
    std::vector<Node> initial_nodes;
    std::vector<Node> goal_nodes;
    std::vector<int> distances;
    explicit HyperabstractionGeneratorResult(
            HyperabstractionGeneratorStatistics&& statistics,
            std::shared_ptr<Hyperabstraction>&& ptr,
            std::vector<Node>&& initial_nodes,
            std::vector<Node>&& goal_nodes,
            std::vector<int>&& distances);
    HyperabstractionGeneratorResult(HyperabstractionGeneratorResult&& other) = default;
};

template<typename Hyperabstraction, typename... AbstractionCollection>
HyperabstractionGeneratorResult<Hyperabstraction>
generate_hyperabstraction(
        const std::shared_ptr<AbstractTask> task,
        bool forward,
        unsigned t_tau_max_nodes,
        unsigned g_tau_max_nodes,
        bool cache_tau_nodes,
        const TransitionPruningMethod& pruning,
        bool prune_inf,
        unsigned transition_enumeration_depth,
        unsigned goal_enumeration_depth,
        unsigned enumeration_max_nodes,
        AbstractionCollection&... collection);

}

#include "generic_hyperabstraction_generator.cc"

#endif
