#include "hyperabstraction_generator.h"
#include "../plugin.h"
#include "../option_parser.h"
#include "../abstract_task.h"
#include "../operator_cost.h"

#include <vector>
#include <string>

namespace hyperabstraction {

HyperabstractionGenerator::HyperabstractionGenerator(const options::Options& opts)
    : c_cache_tau_nodes(opts.get<bool>("cache_tau_nodes"))
    , c_tau_max_nodes((unsigned) std::max(-1, opts.get<int>("tau_max_nodes")))
    , c_tau_max_nodes_goal((unsigned) std::max(-1, opts.get<int>("tau_max_nodes_goal")))
    , c_transition_enumeration_depth((unsigned) std::max(-1, opts.get<int>("transition_enumeration_depth")))
    , c_goal_enumeration_depth((unsigned) std::max(-1, opts.get<int>("goal_enumeration_depth")))
    , c_enumeration_max_nodes((unsigned) std::max(-1, opts.get<int>("enumeration_max_nodes")))
    , c_transition_pruning(TransitionPruningMethod(opts.get_enum("transition_pruning")))
    , m_task(get_task_from_options(opts))
{}

void
HyperabstractionGenerator::add_options_to_parser(options::OptionParser& parser)
{
    parser.add_option<int>("tau_max_nodes", "", "1");
    parser.add_option<int>("tau_max_nodes_goal", "", "-1");
    parser.add_option<bool>("cache_tau_nodes", "", "true");
    parser.add_option<int>("transition_enumeration_depth", "", "1");
    parser.add_option<int>("goal_enumeration_depth", "", "1");
    parser.add_option<int>("enumeration_max_nodes", "", "-1");
    std::vector<std::string> opts({"false", "duplicates", "subsumed" });
    parser.add_enum_option("transition_pruning", opts, "", "duplicates");
    ::add_cost_type_option_to_parser(parser);
    parser.add_option<std::shared_ptr<AbstractTask>>(
        "transform",
        "Optional task transformation for the heuristic. "
        "Currently only adapt_costs is available.",
        OptionParser::NONE);
}


}

static PluginTypePlugin<hyperabstraction::HyperabstractionGenerator>
_type_plugin(
    "HyperabstractionGenerator",
    // TODO: Replace empty string by synopsis for the wiki page.
    "");

