#ifndef HYPERABSTRACTION_GENERATOR_H
#define HYPERABSTRACTION_GENERATOR_H

#include "hyperabstraction.h"
#include "generic_hyperabstraction_generator.h"

class GlobalState;
class AbstractTask;

namespace options {
class Options;
class OptionParser;
}

#include <vector>
#include <memory>

namespace hyperabstraction {

class HyperabstractionGenerator {
public:
    HyperabstractionGenerator(const options::Options& opts);
    virtual HyperabstractionGeneratorResult<Hyperabstraction> generate_backward() = 0;
    virtual HyperabstractionGeneratorResult<LabeledHyperabstraction> generate_backward_labeled() = 0;
    virtual HyperabstractionGeneratorResult<UpdatableHyperabstraction> generate_backward_updatable() = 0;
    virtual HyperabstractionGeneratorResult<Hyperabstraction> generate_forward() = 0;
    virtual HyperabstractionGeneratorResult<LabeledHyperabstraction> generate_forward_labeled() = 0;
    virtual HyperabstractionGeneratorResult<UpdatableHyperabstraction> generate_forward_updatable() = 0;
    virtual void get_nodes(std::vector<Node>& nodes, const GlobalState& state) = 0;
    virtual unsigned num_nodes() const = 0;
    static void add_options_to_parser(options::OptionParser& parser);
protected:
    const bool c_cache_tau_nodes;
    const unsigned c_tau_max_nodes;
    const unsigned c_tau_max_nodes_goal;
    const unsigned c_transition_enumeration_depth;
    const unsigned c_goal_enumeration_depth;
    const unsigned c_enumeration_max_nodes;
    const TransitionPruningMethod c_transition_pruning;
    const std::shared_ptr<AbstractTask> m_task;
};

}

#endif
