#ifndef HYPERABSTRACTION_GENERATOR_STATISTICS_H
#define HYPERABSTRACTION_GENERATOR_STATISTICS_H

#include "../utils/timer.h"
#include <iostream>

namespace hyperabstraction {

enum class TransitionPruningMethod;

class HyperabstractionGeneratorStatistics {
public:
    HyperabstractionGeneratorStatistics();
    HyperabstractionGeneratorStatistics(const HyperabstractionGeneratorStatistics& other) = default;
    HyperabstractionGeneratorStatistics& operator=(const HyperabstractionGeneratorStatistics& other) = default;
    
    unsigned num_tau_transitions() const;
    unsigned num_tau_nodes() const;
    double construction_time() const;

    void print(bool forward, std::ostream& out = std::cout) const;

    void set_transition_pruning_method(const TransitionPruningMethod& method);
    void set_caching_tau_nodes(bool cache);
    void set_tau_max_nodes(const unsigned& t, const unsigned& g);
    void set_prune_inf(bool prune);

    void begin_construction();
    void end_construction(unsigned nodes,
                          unsigned transitions,
                          unsigned hypertransitions);
    void begin_postprocessing();
    void end_postprocessing();
    void inc_tau_transitions(unsigned num = 1);
    void inc_tau_nodes(unsigned num = 1);
    void inc_dinf_nodes(unsigned num = 1);
    void inc_dinf_transitions(unsigned num = 1);
    void inc_pruned_transitions(unsigned num = 1);
    void inc_transitions(const unsigned& tail_size);
private:
    utils::Timer m_construction_timer;
    utils::Timer m_postprocessing_timer;
    unsigned m_num_nodes;
    unsigned m_num_transitions;
    unsigned m_num_hypertransitions;
    unsigned m_num_tau_transitions;
    unsigned m_num_tau_nodes;
    unsigned m_num_dinf_nodes;
    unsigned m_num_dinf_transitions;
    unsigned m_num_pruned_transitions;

    unsigned m_num_t0_transitions;
    unsigned m_num_t1_transitions;
    unsigned m_num_t2_transitions;
    unsigned m_num_t3_transitions;
    unsigned m_num_t4_transitions;
    unsigned m_num_lt4_transitions;

    TransitionPruningMethod m_transition_pruning;
    bool m_cache_tau_nodes;
    unsigned m_t_tau_max_nodes;
    unsigned m_g_tau_max_nodes;
    bool m_prune_inf;
};

}

#endif
