#ifndef HYPERABSTRACTION_TRANSITION_GENERATOR_H
#define HYPERABSTRACTION_TRANSITION_GENERATOR_H

#include "hyperabstraction.h"
#include "hyperabstraction_generator_statistics.h"

#include "../segmented_vector.h"
#include "../utils/ub_tree.h"
#include "../utils/hash.h"

#include <vector>
#include <deque>
#include <unordered_set>
#include <map>

namespace hyperabstraction {

template<typename Abstraction>
class TransitionGeneratorNoPrune {
public:
    TransitionGeneratorNoPrune(Abstraction& abstraction,
                               HyperabstractionGeneratorStatistics&);
    void operator()(const Node& head, const std::vector<Node>& tail, const int& label, const int& cost);
    void finalize() {}
private:
    Abstraction& abstraction;
    HyperabstractionGeneratorStatistics& stats;
};


template<typename Abstraction>
class TransitionGeneratorPruneDuplicate {
public:
    TransitionGeneratorPruneDuplicate(Abstraction& a,
                                      HyperabstractionGeneratorStatistics& stat);
    void operator()(const Node& head, const std::vector<Node>& tail, const int& label, const int& cost);
    void finalize() {}
private:
    using Hasher = utils::CollectionIdHasher<SegmentedVector<std::vector<Node> > >;
    using Comparator = utils::CollectionIdComparator<SegmentedVector<std::vector<Node> > >;
    Abstraction& abstraction;
    SegmentedVector<std::vector<Node> > transitions;
    std::vector<unsigned> transition_ids;
    std::unordered_set<int, Hasher, Comparator> transition_set;
    HyperabstractionGeneratorStatistics& stats;
};


template<typename Abstraction, bool Forward>
class TransitionGeneratorPruneSubsumed {
public:
    TransitionGeneratorPruneSubsumed(Abstraction& abstraction,
                                     HyperabstractionGeneratorStatistics& stat);
    void operator()(const Node& head, const std::vector<Node>& tail, const int& label, const int& cost);
    void finalize();
private:
    Abstraction& abstraction;
    SegmentedVector<std::map<int, utils::UBTreeMap<Node, int>*> > transitions;
    std::deque<utils::UBTreeMap<Node, int> > _local_transitions;
    HyperabstractionGeneratorStatistics& stats;
};

}

#include "hyperabstraction_transition_generator.cc"

#endif
