#include "pdb_hyperabstraction_generator.h"

#include "../global_state.h"
#include "../option_parser.h"
#include "../plugin.h"
#include "../pdbs/pattern_generator.h"
#include "../pdbs/pattern_collection_information.h"

#include "../abstract_task.h"
#include "../task_tools.h"

#include <memory>
#include <cassert>

#ifndef NDEBUG
#include "../pdbs/pattern_database.h"
#include <algorithm>
#include <fstream>
#include <sstream>
#endif

namespace hyperabstraction {

PDBHyperabstractionGenerator::PDBHyperabstractionGenerator(const options::Options& opts)
    : HyperabstractionGenerator(opts)
{
    std::shared_ptr<pdbs::PatternCollectionGenerator> gen =
        opts.get<std::shared_ptr<pdbs::PatternCollectionGenerator> >("patterns");
    pdbs::PatternCollectionInformation info = gen->generate(m_task);
    std::shared_ptr<pdbs::PatternCollection> collection = nullptr;
    if (opts.get<bool>("ignore_subsumed_patterns")) {
        collection = std::make_shared<pdbs::PatternCollection>(std::move(
                    info.get_non_subsumed_patterns()));
    } else {
        collection = info.get_patterns();
    }
    m_node_mappers.reserve(collection->size());
    m_num_nodes = 0;
    for (unsigned i = 0; i < collection->size(); i++) {
        //if (i != 0) continue;
        //std::cout << "pattern[" << i << "]:" << std::endl;
        //for (unsigned j = 0; j < collection->at(i).size(); j++) {
        //    std::cout << m_task->get_fact_name(FactPair(collection->at(i)[j], 0)) << std::endl;
        //}
        m_node_mappers.emplace_back(m_task,
                                    collection->at(i),
                                    m_num_nodes,
                                    opts.get<bool>("enumerate_tail"));
        m_num_nodes += m_node_mappers.back().num_nodes();
    }

#if 0
#ifndef NDEBUG
    std::cout << std::endl;
    unsigned offset = 0;
    std::vector<int> pre(m_task->get_num_variables(), -1);
    std::vector<int> post(m_task->get_num_variables(), -1);
    std::shared_ptr<pdbs::PDBCollection> pdbs = info.get_pdbs();
    for (unsigned i = 0; i < collection->size(); i++) {
        // if (i != 0) continue;
        std::shared_ptr<pdbs::PatternDatabase> pdb = pdbs->at(i);
        const std::vector<int>& d = pdb->get_distances();
        for (unsigned node = 0; node < m_node_mappers[i].num_nodes(); node++) {
            assert(node < d.size());
            std::cout << "d[" << (node+offset) << "] = " << d[node] << std::endl;
        }

        // std::ostringstream fn; fn << "pdb" << i << ".dot";
        // std::ofstream out; out.open(fn.str());
        // out << "digraph {" << std::endl;
        // for (unsigned node = 0; node < m_node_mappers[i].num_nodes(); node++) {
        //     out << "  node" << (node + offset)
        //         << " [shape=circle, label=\"" << (node + offset) << "\"];"
        //         << std::endl;
        // }
        // for (int op = 0; op < m_task->get_num_operators(); op++) {
        //     std::fill(pre.begin(), pre.end(), -1);
        //     std::fill(post.begin(), post.end(), -1);
        //     for (int j = 0; j < m_task->get_num_operator_effects(op, false); j++) {
        //         FactPair p = m_task->get_operator_effect(op, j, false);
        //         post[p.var] = p.value;
        //     }
        //     for (int j = 0; j < m_task->get_num_operator_preconditions(op, false); j++) {
        //         FactPair p = m_task->get_operator_precondition(op, j, false);
        //         pre[p.var] = p.value;
        //         if (post[p.var] == -1) {
        //             post[p.var] = p.value;
        //         }
        //     }
        //     auto callback = [&](const Node& node_id, const std::vector<int>& progression) {
        //         std::vector<Node> successors;
        //         m_node_mappers[i].get_compatible_nodes(successors, progression);
        //         for (unsigned j = 0; j < successors.size(); j++) {
        //             out << "  node" << node_id << " -> " << " node" << successors[j] << " [label=\"" << op << "\"];" << std::endl;
        //         }
        //     };
        //     m_node_mappers[i].for_each_intersecting_node(pre, post, callback);
        // }
        // out << "}" << std::endl;
        // out.close();

        offset += m_node_mappers[i].num_nodes();
    }
    std::cout << std::endl;

    // offset = 0;
    // for (unsigned i = 0; i < collection->size(); i++) {
    //     // if (i != 0) continue;
    //     std::cout << "PDB[" << i << "] = {" << (offset) << ", ..., " << (offset + m_node_mappers[i].num_nodes()) << "}" << std::endl;
    //     offset += m_node_mappers[i].num_nodes();
    // }
#endif
#endif
}

HyperabstractionGeneratorResult<Hyperabstraction>
PDBHyperabstractionGenerator::generate_backward()
{
    return generate<Hyperabstraction, false>();
}

HyperabstractionGeneratorResult<LabeledHyperabstraction>
PDBHyperabstractionGenerator::generate_backward_labeled()
{
    return generate<LabeledHyperabstraction, false>();
}

HyperabstractionGeneratorResult<UpdatableHyperabstraction>
PDBHyperabstractionGenerator::generate_backward_updatable()
{
    return generate<UpdatableHyperabstraction, false>();
}

HyperabstractionGeneratorResult<Hyperabstraction>
PDBHyperabstractionGenerator::generate_forward()
{
    return generate<Hyperabstraction, true>();
}

HyperabstractionGeneratorResult<LabeledHyperabstraction>
PDBHyperabstractionGenerator::generate_forward_labeled()
{
    return generate<LabeledHyperabstraction, true>();
}

HyperabstractionGeneratorResult<UpdatableHyperabstraction>
PDBHyperabstractionGenerator::generate_forward_updatable()
{
    return generate<UpdatableHyperabstraction, true>();
}

void
PDBHyperabstractionGenerator::get_nodes(std::vector<Node>& nodes,
                                        const GlobalState& state)
{
    for (int i = m_node_mappers.size() - 1; i >= 0; i--) {
        nodes.push_back(m_node_mappers[i].get_node(state));
    }
}

unsigned
PDBHyperabstractionGenerator::num_nodes() const
{
    unsigned res = 0;
    for (int i = m_node_mappers.size() - 1; i >= 0; i--) {
        res += m_node_mappers[i].num_nodes();
    }
    return res;
}

void
PDBHyperabstractionGenerator::add_options_to_parser(options::OptionParser& parser)
{
    parser.add_option<std::shared_ptr<pdbs::PatternCollectionGenerator> >(
            "patterns",
            "",
            "systematic(pattern_max_size=1, only_interesting_patterns=true)");
    parser.add_option<bool>("enumerate_tail", "", "false");
    parser.add_option<bool>("ignore_subsumed_patterns", "", "false");
    HyperabstractionGenerator::add_options_to_parser(parser);
}

}




namespace hyperabstraction {

PDBNodeMapper::PDBNodeMapper(const std::shared_ptr<AbstractTask> task,
                              const pdbs::Pattern& pattern,
                              unsigned offset,
                              bool enumerate_tail)
    : c_enumerate_tail(enumerate_tail)
    , m_task(task)
    , m_pattern(pattern)
    , m_sizes(pattern.size())
    , m_offset(offset)
{
    m_num_nodes = 1;
    for (unsigned i = 0; i < pattern.size(); i++) {
        unsigned size = m_task->get_variable_domain_size(pattern[i]);
        m_sizes[i] = m_num_nodes;
        m_num_nodes *= size;
    }
}

bool
PDBNodeMapper::get_compatible_nodes(
        std::vector<Node>& result,
        const std::vector<int>& partial_state,
        unsigned max_nodes) const
{
    if (max_nodes < m_num_nodes
            && get_num_compatible_nodes(partial_state, max_nodes) > max_nodes) {
        return false;
    }
    for_each_compatible_node(
            partial_state,
            [&result](const Node& node) { result.push_back(node); },
            m_pattern.size() - 1,
            m_offset);
    assert(result.size() <= max_nodes);
    return true;
}

bool
PDBNodeMapper::get_compatible_nodes(
        std::vector<Node>& result,
        const cegar::Domains& partial_state,
        unsigned max_nodes) const
{
    if (max_nodes < m_num_nodes
            && get_num_compatible_nodes(partial_state, max_nodes) > max_nodes) {
        return false;
    }
    for_each_compatible_node(
            partial_state,
            [&result](const Node& node) { result.push_back(node); },
            m_pattern.size() - 1,
            m_offset);
    assert(result.size() <= max_nodes);
    return true;
}

unsigned
PDBNodeMapper::get_num_compatible_nodes(
        const std::vector<int>& partial_state,
        unsigned ) const
{
    unsigned num = 1;
    for (int i = m_pattern.size() - 1; i >= 0; i--) {
        if (partial_state[m_pattern[i]] == -1) {
            num *= m_task->get_variable_domain_size(m_pattern[i]);
        }
    }
    return num;
}

unsigned
PDBNodeMapper::get_num_compatible_nodes(
        const cegar::Domains& partial_state,
        unsigned ) const
{
    unsigned num = 1;
    for (int i = m_pattern.size() - 1; i >= 0; i--) {
        num *= partial_state.count(m_pattern[i]);
    }
    return num;
}

std::vector<int>
PDBNodeMapper::compute_intersection(
        Node node,
        const std::vector<int>& state) const
{
    // auto dumpv = [](const auto& v) {
    //     std::cout << "[";
    //     bool sep = false;
    //     for (unsigned i = 0; i < v.size(); i++) {
    //         // if ((unsigned) v[i] == (unsigned) -1) continue;
    //         if (sep) std::cout << ", ";
    //         if (v.size() > 5)  std::cout << i << ":";
    //         std::cout << v[i];
    //         sep = true;
    //     }
    //     std::cout << "]";
    // };
    // std::cout << "intersection(offset=" << m_offset << " node=" << (node - m_offset) << ", pattern=";
    // dumpv(m_pattern);
    // std::cout << ", state=";
    // dumpv(state);
    // std::cout << std::endl;

    
    node -= m_offset;
    std::vector<int> result(state);
    for (int i = m_pattern.size() - 1; i >= 0; i--) {
        int val = node / m_sizes[i];
        node -= val * m_sizes[i];
        assert(result[m_pattern[i]] == -1 || result[m_pattern[i]] == val);
        result[m_pattern[i]] = val;
    }
    assert(node == 0);

    // std::cout << "  -> result = ";
    // dumpv(result);
    // std::cout << std::endl;

    return std::move(result);
}

cegar::Domains
PDBNodeMapper::compute_intersection(
        Node node,
        const cegar::Domains& cegar_state) const
{
    node -= m_offset;
    cegar::Domains result(cegar_state);
    for (int i = m_pattern.size() - 1; i >= 0; i--) {
        int val = node / m_sizes[i];
        assert(result.test(m_pattern[i], val));
        result.set_single_value(m_pattern[i], val);
        node -= val * m_sizes[i];
    }
    assert(node == 0);
    return std::move(result);
}

Node
PDBNodeMapper::get_node(const std::vector<int>& partial_state) const
{
    Node res = m_offset;
    for (int i = m_pattern.size() - 1; i >= 0; i--) {
        int val = partial_state[m_pattern[i]];
        if (val < 0) {
            return no_node;
        }
        res += val * m_sizes[i];
    }
    return res;
}

Node
PDBNodeMapper::get_node(const GlobalState& state) const
{
    Node res = m_offset;
    for (int i = m_pattern.size() - 1; i >= 0; i--) {
        res += state[m_pattern[i]] * m_sizes[i];
    }
    return res;
}

unsigned
PDBNodeMapper::num_nodes() const
{
    return m_num_nodes;
}

}

static std::shared_ptr<hyperabstraction::HyperabstractionGenerator>
_parse(options::OptionParser& parser)
{
    hyperabstraction::PDBHyperabstractionGenerator::add_options_to_parser(parser);
    options::Options opts = parser.parse();
    if (!parser.dry_run()) {
        return std::make_shared<hyperabstraction::PDBHyperabstractionGenerator>(opts);
    }
    return nullptr;
}

static PluginShared<hyperabstraction::HyperabstractionGenerator> _plugin("pdb", _parse);
