#ifndef HYPERABSTRACTION_UTILS_H
#define HYPERABSTRACTION_UTILS_H

#include "hyperabstraction.h"
#include "hyperabstraction_generator.h"

#include <memory>

namespace options {
class Options;
class OptionParser;
}

namespace hyperabstraction {

int get_max_distance(const std::vector<int>& distances,
                     const std::vector<Node>& nodes);

bool is_reachable(const std::vector<bool>& reachable,
                  const std::vector<Node>& nodes);

template<typename H1, typename H2>
int compute_bidirectional_distances(
        std::vector<int>& distances,
        const std::vector<Node>& init_fw,
        const std::vector<Node>& goal_bw,
        const std::vector<Node>& state,
        const unsigned& num_shared_nodes,
        H1& abstraction_fw,
        H2& abstraction_bw);

template<typename H1, typename H2>
bool compute_bidirectional_reachability(
        std::vector<bool>& reachable,
        const std::vector<Node>& init_fw,
        const std::vector<Node>& goal_bw,
        const std::vector<Node>& state,
        const unsigned& num_shared_nodes,
        H1& abstraction_fw,
        H2& abstraction_bw);

void add_hyperabstraction_options_to_parser(options::OptionParser& parser);

std::shared_ptr<HyperabstractionGenerator> get_hyperabstraction_generator_from_options(const options::Options& opts);

HyperabstractionGeneratorResult<Hyperabstraction>
get_hyperabstraction_from_options(
        const options::Options& opts,
        std::shared_ptr<HyperabstractionGenerator> generator,
        bool forward);

HyperabstractionGeneratorResult<LabeledHyperabstraction>
get_labeled_hyperabstraction_from_options(
        const options::Options& opts,
        std::shared_ptr<HyperabstractionGenerator> generator,
        bool forward);

std::pair<HyperabstractionGeneratorResult<Hyperabstraction>, HyperabstractionGeneratorResult<Hyperabstraction> >
get_bidirectional_hyperabstractions_from_options(
        const options::Options& opts,
        std::shared_ptr<HyperabstractionGenerator> generator);

}

#include "utils.cc"

#endif
