#ifndef OPTION_PARSER_H
#define OPTION_PARSER_H

/* We use this temporary stub to avoid merge conflicts while we move the
   option parser code into a subdirectory. */
#include "options/errors.h"
#include "options/option_parser.h"
#include "options/synergy.h"

using options::ArgError;
using options::Bounds;
using options::OptionParser;
using options::Options;
using options::ParseError;
using options::Synergy;

namespace options {

template<typename BaseType, typename InstanceType>
BaseType*
parse(OptionParser& parser)
{
    InstanceType::add_options_to_parser(parser);
    Options opts = parser.parse();
    if (!parser.dry_run()) {
        return new InstanceType(opts);
    }
    return NULL;
}

template<typename BaseType, typename InstanceType>
std::shared_ptr<BaseType>
parse_shared(OptionParser& parser)
{
    InstanceType::add_options_to_parser(parser);
    Options opts = parser.parse();
    if (!parser.dry_run()) {
        return std::make_shared<InstanceType>(opts);
    }
    return nullptr;
}

}

#endif
