#include "max_pdb_heuristic.h"
#include "pattern_database.h"
#include "pattern_collection_information.h"
#include "pattern_generator.h"

#include "../option_parser.h"
#include "../plugin.h"

#include <limits>

namespace pdbs {

MaxPDBHeuristic::MaxPDBHeuristic(const options::Options& opts)
    : Heuristic(opts)
    , m_pdb_collection(nullptr)
{
    std::shared_ptr<PatternCollectionGenerator> generator =
        opts.get<std::shared_ptr<PatternCollectionGenerator>>("patterns");
    m_pdb_collection = generator->generate(task).get_pdbs();
}

int
MaxPDBHeuristic::compute_heuristic(const GlobalState& state)
{
    State s = convert_global_state(state);
    int value = 0;
    for (int i = m_pdb_collection->size() - 1; i >= 0; i--) {
        value = std::max(value, m_pdb_collection->at(i)->get_value(s));
    }
    return value == std::numeric_limits<int>::max() ? DEAD_END : value;
}

void
MaxPDBHeuristic::add_options_to_parser(options::OptionParser& parser)
{
    parser.add_option<std::shared_ptr<PatternCollectionGenerator>>("patterns");
    Heuristic::add_options_to_parser(parser);
}

}

static Plugin<Heuristic> _plugin("max_pdb",
                                 options::parse<Heuristic, pdbs::MaxPDBHeuristic>);
