#ifndef PDBS_PATTERN_COLLECTION_GENERATOR_RANDOM_H
#define PDBS_PATTERN_COLLECTION_GENERATOR_RANDOM_H

#include "pattern_generator.h"
#include "../option_parser.h"
#include "../utils/rng.h"
#include "../utils/countdown_timer.h"

namespace pdbs {

class PatternCollectionGeneratorRandom : public PatternCollectionGenerator {
public:
    PatternCollectionGeneratorRandom(const options::Options& opts);
    void shuffle(std::vector<int>& set);
    int sample(std::vector<int>& connected,
               std::vector<int>& goal,
               std::vector<int>& remaining,
               bool start);
    static void add_options_to_parser(options::OptionParser& parser);
protected:
    virtual PatternCollectionInformation generate_collection(
            std::shared_ptr<AbstractTask> task) override;
private:
    const unsigned c_max_states_per_pattern;
    const unsigned c_max_states_overall;
    const bool c_p_strict;
    const double c_p_goal;
    const double c_p_connectivity;
    const bool c_allow_subsets;
    utils::CountdownTimer m_timeout;
    utils::RandomNumberGenerator m_rng;
};

}

#endif
