#include "perfect_heuristic.h"

#include "sym_variables.h"
#include "uniform_cost_search.h"
#include "original_state_space.h"

#include "sym_enums.h"
#include "../option_parser.h"
#include "../plugin.h"
#include "../globals.h"
#include "../utils/debug_macros.h"


#include <cassert>
#include <limits>
#include <memory>
#include <vector>

using namespace std;

namespace symbolic {

PerfectHeuristic::PerfectHeuristic()
  : SymController() {

    auto state_space = make_shared<OriginalStateSpace> (vars.get(), mgrParams, 
							OperatorCostFunction::get_cost_function());

    auto uc_search = make_unique<UniformCostSearch> (this, searchParams);
    uc_search->init(state_space, false);
    while (!uc_search->finished()) {
	if(!uc_search->step()) break;
    }
    perfect_heuristic = make_unique<ADD>(uc_search->getClosed()->getHeuristic());
}


int PerfectHeuristic::compute_heuristic(const GlobalState &state) {
    int * inputs = vars->getBinaryDescription(state);
    
    ADD evalNode = perfect_heuristic->Eval(inputs);
    int abs_cost = Cudd_V(evalNode.getRegularNode());
    
    if (abs_cost == -1) return std::numeric_limits<int>::max();
    return abs_cost;
}


}
