#ifdef UTILS_UB_TREE_H

#include <deque>

namespace std
{
// template<typename T1, typename T2>
// static ostream &operator<<(ostream &out, const std::pair<T1, T2> &p) {
//     out << "(" << p.first << "," << p.second << ")";
//     return out;
// }
//
// template<typename T>
// static ostream &operator<<(ostream &out, const std::vector<T> &vals) {
//     out << "[";
//     bool sep = false;
//     for (const auto &t : vals) {
//         out << (sep ? ", " : "") << t;
//         sep = true;
//     }
//     out << "]";
//     return out;
// }
}

namespace utils
{

template<typename K, typename V, typename Less, typename EqualTo>
UBTreeMap<K, V, Less, EqualTo>::const_iterator::const_iterator()
{
}

template<typename K, typename V, typename Less, typename EqualTo>
UBTreeMap<K, V, Less, EqualTo>::const_iterator::const_iterator(const Node* node)
{
    if (node->m != NULL || !node->s.empty()) {
        q.emplace_back(node);
    }
}

template<typename K, typename V, typename Less, typename EqualTo>
UBTreeMap<K, V, Less, EqualTo>::const_iterator::const_iterator(const const_iterator& other) :
    q(other.q)
{
}

template<typename K, typename V, typename Less, typename EqualTo>
UBTreeMap<K, V, Less, EqualTo>::const_iterator::const_iterator(const const_iterator&& other) :
    q(std::move(other.q))
{
}

template<typename K, typename V, typename Less, typename EqualTo>
const K& UBTreeMap<K, V, Less, EqualTo>::const_iterator::key() const
{
    return *(q.back().key);
}

template<typename K, typename V, typename Less, typename EqualTo>
const V& UBTreeMap<K, V, Less, EqualTo>::const_iterator::value() const
{
    return *(q.back().node->m);
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::const_iterator::is_set_completed() const
{
    return q.back().node->m != NULL;
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::const_iterator::is_root() const
{
    return q.back().key == NULL;
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::const_iterator::is_terminal() const
{
    if (q.empty()) { return false; }
    auto &pos = q.back();
    return pos.i >= pos.node->s.size();
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::const_iterator::operator==(const const_iterator& other) const
{
    return other.q.empty() == q.empty()
        && (q.empty() || (other.q.back() == q.back()));
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::const_iterator::operator!=(const const_iterator& other) const
{
    return !(*this == other);
}

template<typename K, typename V, typename Less, typename EqualTo>
typename UBTreeMap<K, V, Less, EqualTo>::const_iterator& UBTreeMap<K, V, Less, EqualTo>::const_iterator::operator++(int)
{
    pos_t &pos = q.back();
    if (pos.i >= pos.node->s.size()) {
        skip();
    } else {
        const auto &succ = pos.node->s[pos.i];
        q.emplace_back(&succ.first, succ.second);
    }
    return *this;
}

template<typename K, typename V, typename Less, typename EqualTo>
typename UBTreeMap<K, V, Less, EqualTo>::const_iterator& UBTreeMap<K, V, Less, EqualTo>::const_iterator::skip()
{
    q.pop_back();
    if (!q.empty()) {
        pos_t &pos = q.back();
        pos.i++;
        // const auto &succs = pos.node->s;
        // if (++pos.i < succs.size()) {
        //     q.emplace_back(&succs[pos.i].first, succs[pos.i].second);
        // }
    }
    return *this;
}

template<typename K, typename V, typename Less, typename EqualTo>
typename UBTreeMap<K, V, Less, EqualTo>::Node* UBTreeMap<K, V, Less, EqualTo>::get(Node* node, const K& key)
{
    NodeList &succs = node->s;
    for (unsigned i = 0; i < succs.size(); i++) {
        if (equal_to(succs[i].first, key)) {
            return succs[i].second;
        }
    }
    return NULL;
}

template<typename K, typename V, typename Less, typename EqualTo>
typename UBTreeMap<K, V, Less, EqualTo>::Node* UBTreeMap<K, V, Less, EqualTo>::get(Node* node, const std::vector<K>& keys)
{
    for (unsigned i = 0; node && i < keys.size(); i++) {
        node = get(node, keys[i]);
    }
    return node;
}

template<typename K, typename V, typename Less, typename EqualTo>
const typename UBTreeMap<K, V, Less, EqualTo>::Node* UBTreeMap<K, V, Less, EqualTo>::get(const Node* node, const K& key)
{
    const NodeList &succs = node->s;
    for (unsigned i = 0; i < succs.size(); i++) {
        if (equal_to(succs[i].first, key)) {
            return succs[i].second;
        }
    }
    return NULL;
}

template<typename K, typename V, typename Less, typename EqualTo>
const typename UBTreeMap<K, V, Less, EqualTo>::Node* UBTreeMap<K, V, Less, EqualTo>::get(const Node* node, const std::vector<K>& keys)
{
    for (unsigned i = 0; node && i < keys.size(); i++) {
        node = get(node, keys[i]);
    }
    return node;
}

template<typename K, typename V, typename Less, typename EqualTo>
const V* UBTreeMap<K, V, Less, EqualTo>::get_any_subset(const Node* node, const std::vector<K>& keys, unsigned i)
{
    if (node->m) {
        return node->m;
    }
    const V* res = NULL;
    if (i < keys.size()) {
        const NodeList& nodes = node->s;
        unsigned k;
        for (unsigned j = 0; !res && j < nodes.size(); j++) {
            k = i;
            while (k < keys.size() && less(keys[k], nodes[j].first)) {
                k++;
            }
            if (k < keys.size() && equal_to(keys[k], nodes[j].first)) {
                res = get_any_subset(nodes[j].second, keys, k + 1);
            }
        }
    }
    return res;
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::contains_superset(const Node* node, const std::vector<K>& keys, unsigned i)
{
    if (i == keys.size()) {
        return true;
    }
    if (i < keys.size()) {
        const NodeList& nodes = node->s;
        for (unsigned j = 0; j < nodes.size(); j++) {
            if (equal_to(keys[i], nodes[j].first)) {
                if (contains_superset(nodes[j].second, keys, i + 1)) {
                    return true;
                }
            } else if (less(nodes[j].first, keys[i])) {
                if (contains_superset(nodes[j].second, keys, i)) {
                    return true;
                }
            }
        }
    }
    return false;
}

template<typename K, typename V, typename Less, typename EqualTo>
template<typename F>
void UBTreeMap<K, V, Less, EqualTo>::for_every_subset_do(const Node* node, const std::vector<K>& keys, unsigned i, const F &fun)
{
    // std::cout << keys << "; i=" << i << std::endl;
    if (node->m) {
    //     std::cout << "hit -> " << (*node->m) << std::endl;
         fun(*node->m);
    }
    if (i < keys.size()) {
        const NodeList& nodes = node->s;
        unsigned k;
        for (unsigned j = 0; j < nodes.size(); j++) {
            k = i;
            while (k < keys.size() && less(keys[k], nodes[j].first)) {
                k++;
            }
            if (k < keys.size() && equal_to(keys[k], nodes[j].first)) {
                for_every_subset_do(nodes[j].second, keys, k + 1, fun);
            }
        }
    }
}

template<typename K, typename V, typename Less, typename EqualTo>
template<typename F>
void UBTreeMap<K, V, Less, EqualTo>::for_every_superset_do(const Node* node, const std::vector<K>& keys, unsigned i, const F &fun)
{
    if (i == keys.size() && node->m) {
        fun(*node->m);
    }
    const NodeList& nodes = node->s;
    unsigned k;
    for (unsigned j = 0; j < nodes.size(); j++) {
        const auto &succ = nodes[j];
        k = i;
        if (i != keys.size()) {
            if (keys[i] == succ.first) {
                k++;
            } else if (keys[i] < succ.first) {
                continue;
            }
        } 
        for_every_superset_do(succ.second, keys, k, fun);
    }
}

template<typename K, typename V, typename Less, typename EqualTo>
template<typename F>
int UBTreeMap<K, V, Less, EqualTo>::remove(Node* node, const std::vector<K>& keys, unsigned i, const F& fun)
{
    if (i == keys.size()) {
        int res = 0;
        if (node->m != NULL) {
            res = 1;
            if (fun(node->m)) {
                node->m = NULL;
                return node->s.empty() ? -1 : 1; 
            }
        }
        return res;
    }
    int r;
    NodeList& nodes = node->s;
    for (unsigned j = 0; j < nodes.size(); j++) {
        if (equal_to(keys[i], nodes[j].first)) {
            r = remove(nodes[j].second, keys, i + 1, fun);
            if (r == -1) {
                if (j + 1 < nodes.size()) {
                    nodes[j].swap(nodes.back());
                }
                nodes.pop_back();
                return node->m == NULL && nodes.empty() ? -1 : 1;
            }
            return r;
        } 
    }
    return 0;
}

template<typename K, typename V, typename Less, typename EqualTo>
typename UBTreeMap<K, V, Less, EqualTo>::Node* UBTreeMap<K, V, Less, EqualTo>::get_or_create(Node* node, const K& key)
{
    NodeList& succs = node->s;
    for (unsigned i = 0; i < succs.size(); i++) {
        if (equal_to(succs[i].first, key)) {
            return succs[i].second;
        }
    }
    m_nodes.push_back(Node());
    succs.emplace_back(key, &m_nodes[m_num_nodes++]);
    return succs.back().second;
}



template<typename K, typename V, typename Less, typename EqualTo>
UBTreeMap<K, V, Less, EqualTo>::UBTreeMap()
{
    m_num_nodes = 0;
    m_num_leafs = 0;
}

template<typename K, typename V, typename Less, typename EqualTo>
UBTreeMap<K, V, Less, EqualTo>::UBTreeMap(UBTreeMap &&
#ifndef NDEBUG
        other
#endif
        )
{
    assert(other.m_nodes.size() == 0);
    assert(other.m_leaf_data.size() == 0);
    assert(other.m_num_nodes == 0);
    assert(other.m_num_leafs == 0);
    m_num_nodes = 0;
    m_num_leafs = 0;
}

template<typename K, typename V, typename Less, typename EqualTo>
std::pair<V*, bool> UBTreeMap<K, V, Less, EqualTo>::insert(const std::vector<K>& key, const V& v)
{
    Node *node = &m_root;
    for (unsigned i = 0; i < key.size(); i++) {
        node = get_or_create(node, key[i]);
    }
    if (node->m == NULL) {
        m_leaf_data.push_back(v);
        node->m = &m_leaf_data[m_num_leafs++];
        return std::pair<V*, bool>(node->m, true);
    }
    return std::pair<V*, bool>(node->m, false);
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::remove(const std::vector<K>& keys)
{
    return remove(&m_root, keys, 0, [](const V*){return true;}) != 0;
}

template<typename K, typename V, typename Less, typename EqualTo>
template<typename F>
bool UBTreeMap<K, V, Less, EqualTo>::remove(const std::vector<K>& keys, const F& fun)
{
    return remove(&m_root, keys, 0, fun) != 0;
}

template<typename K, typename V, typename Less, typename EqualTo>
const V* UBTreeMap<K, V, Less, EqualTo>::get(const std::vector<K>& keys) const
{
    const Node *node = get(&m_root, keys);
    return node == NULL ? NULL : node->m;
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::contains(const std::vector<K>& keys) const
{
    return get(keys) != NULL;
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::contains_subset(const std::vector<K>& keys) const
{
    const V *val = get_any_subset(&m_root, keys, 0);
    return val != NULL;
}

template<typename K, typename V, typename Less, typename EqualTo>
bool UBTreeMap<K, V, Less, EqualTo>::contains_superset(const std::vector<K>& keys) const
{
    return contains_superset(&m_root, keys, 0);
}

template<typename K, typename V, typename Less, typename EqualTo>
template<typename F>
void UBTreeMap<K, V, Less, EqualTo>::for_every_subset_do(const std::vector<K>& key,
                                                         const F& fun) const
{
    for_every_subset_do(&m_root, key, 0, fun);
}

template<typename K, typename V, typename Less, typename EqualTo>
void UBTreeMap<K, V, Less, EqualTo>::get_values_for_all_subsets(
        const std::vector<K>& key,
        std::vector<V>& result) const
{
    for_every_subset_do(&m_root, key, 0, [&result](const V& v){result.push_back(v);});
}

template<typename K, typename V, typename Less, typename EqualTo>
template<typename F>
void UBTreeMap<K, V, Less, EqualTo>::for_every_superset_do(const std::vector<K>& key,
                                                           const F& fun) const
{
    for_every_superset_do(&m_root, key, 0, fun);
}

template<typename K, typename V, typename Less, typename EqualTo>
void UBTreeMap<K, V, Less, EqualTo>::clear()
{
    m_num_nodes = 0;
    m_num_leafs = 0;
    m_root.s.clear();
    m_root.m = NULL;
    m_nodes.clear();
    m_leaf_data.clear();
}

template<typename K, typename V, typename Less, typename EqualTo>
size_t UBTreeMap<K, V, Less, EqualTo>::size() const
{
    return m_num_leafs;
}

template<typename K, typename V, typename Less, typename EqualTo>
typename UBTreeMap<K, V, Less, EqualTo>::const_iterator UBTreeMap<K, V, Less, EqualTo>::begin() const
{
    return const_iterator(&m_root);
}

template<typename K, typename V, typename Less, typename EqualTo>
typename UBTreeMap<K, V, Less, EqualTo>::const_iterator UBTreeMap<K, V, Less, EqualTo>::end() const
{
    return const_iterator();
}

template<typename K, typename V, typename Less, typename EqualTo>
void UBTreeMap<K, V, Less, EqualTo>::graphviz(std::ostream &out) const
{
    out << "digraph {" << std::endl;
    std::deque<std::pair<int, const Node *> > nodes;
    nodes.emplace_back(0, &m_root);
    int nodeid = 1;
    out << "node0 [label=\"root";
    if (m_root.m != NULL) {
        out << " " << (*m_root.m);
    }
    out << "\"];" << std::endl;
    while (!nodes.empty()) {
        std::pair<int, const Node *> i = nodes.front();
        nodes.pop_front();
        for (const auto &succ : i.second->s) {
            out << "node" << nodeid << " [label=\"" << nodeid;
            if (succ.second->m != NULL) {
                out << " " << *succ.second->m;
            }
            out << "\"];" << std::endl;
            out << "node" << i.first << " -> node" << nodeid
                << " [label=\"" << succ.first << "\"];" << std::endl;
            nodes.emplace_back(nodeid, succ.second);
            nodeid++;
        }
    }
    out << "}" << std::endl;
}


}

#endif
